@@ -30,10 +30,7 @@ use {
         get_context_value, optional_dict_arg, optional_str_arg, EnvironmentContext, ResolvedTarget,
         ResolvedTargetValue, RunMode,
     },
-    std::{
-        convert::TryFrom,
-        path::{Path, PathBuf},
-    },
+    std::path::{Path, PathBuf},
     tugger_code_signing::SigningDestination,
     tugger_file_manifest::{FileEntry, FileManifest},
     tugger_wix::{WiXInstallerBuilder, WiXSimpleMsiBuilder, WxsBuilder},
